@@ -112,6 +112,19 @@ test!(
     "#
 );
 
+test!(
+    indices_with_empty_string,
+    r#"
+    indices("")
+    "#,
+    r#"
+    "abcde"
+    "#,
+    r#"
+    []
+    "#
+);
+
 test!(
     indices_with_empty_array,
     r#"
@@ -125,6 +138,19 @@ test!(
     "#
 );
 
+test!(
+    indices_against_null,
+    r#"
+    indices(0)
+    "#,
+    r#"
+    null
+    "#,
+    r#"
+    null
+    "#
+);
+
 test!(
     limit_infinite_stream,
     r#"
@@ -360,3 +386,97 @@ test!(
     {"x": null, "y": true, "z": false, "true": 1, "false": 2, "null": 3, "try": 4, "else": 5}
     "#
 );
+
+test!(
+    object_indexing_optional,
+    r#"
+    [1,{x:2},3,{x:4},5] | [.[].x?]
+    "#,
+    r#"
+    null
+    "#,
+    r#"
+    [2,4]
+    "#
+);
+
+test!(
+    multiple_optional_operators,
+    r#"
+    .x??, .
+    "#,
+    r#"
+    0
+    "#,
+    r#"
+    0
+    "#
+);
+
+test!(
+    error_against_null_backtrack,
+    r#"
+    [0, error, 1], (.x, error(null), .y) = 1
+    "#,
+    r#"
+    null
+    "#,
+    r#"
+    [0,1]
+    {"x":1,"y":1}
+    "#
+);
+
+test!(
+    try_catch_value,
+    r#"
+    try error({ x: 0 }) catch .
+    "#,
+    r#"
+    null
+    "#,
+    r#"
+    {"x":0}
+    "#
+);
+
+test!(
+    recurse_label1,
+    r#"
+    isempty(isempty(empty))
+    "#,
+    r#"
+    null
+    "#,
+    r#"
+    false
+    "#
+);
+
+test!(
+    recurse_label2,
+    r#"
+    first((0, 0) | first(0))
+    "#,
+    r#"
+    null
+    "#,
+    r#"
+    0
+    "#
+);
+
+test!(
+    label_prevent_tailrec,
+    r#"
+    def f: label $x | if . < 3 then ., (. + 1 | f, break $x, .) else . end; 1 | f
+    "#,
+    r#"
+    null
+    "#,
+    r#"
+    1
+    2
+    3
+    "#
+);
